<span class="abx-output-icon abx-output-icon--archivewebpage" title="ArchiveWebPage (WACZ)"><svg width="16" height="16" viewBox="0 0 24 24" aria-hidden="true" focusable="false" fill="none" stroke="currentColor" stroke-width="1.8" stroke-linecap="round" stroke-linejoin="round"><rect x="3" y="7" width="18" height="13" rx="1.5"/><path d="M3 11h18"/><path d="M6 7V5a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v2"/><circle cx="8" cy="15.5" r="1" fill="currentColor"/><path d="M11 15.5h7"/></svg></span>
