<span class="abx-output-icon abx-output-icon--twocaptcha" title="2Captcha"><svg width="16" height="16" viewBox="0 0 24 24" aria-hidden="true" focusable="false" fill="none" stroke="currentColor" stroke-width="1.8" stroke-linecap="round" stroke-linejoin="round"><rect x="4" y="5" width="16" height="14" rx="2"/><path d="M4 9h16"/><rect x="7" y="12" width="2.5" height="2.5" rx=".5"/><rect x="11.25" y="12" width="2.5" height="2.5" rx=".5"/><path d="M16 13.25l1.5 1.5 2.5-3"/></svg></span>
