<span class="abx-output-icon abx-output-icon--apt" title="APT"><svg width="16" height="16" viewBox="0 0 24 24" aria-hidden="true" focusable="false" fill="none" stroke="currentColor" stroke-width="1.8" stroke-linecap="round" stroke-linejoin="round"><path d="M12 3l7 4-7 4-7-4z"/><path d="M5 7v8l7 4 7-4V7"/><path d="M12 11v8"/></svg></span>
